@@ -20,8 +20,8 @@ use crate::cast::AsArray;
 use crate::iterator::ArrayIter;
 use crate::types::*;
 use crate::{
-    make_array, Array, ArrayAccessor, ArrayRef, ArrowPrimitiveType, PrimitiveArray,
-    StringArray,
+    make_array, Array, ArrayAccessor, ArrayRef, ArrowNativeTypeOp, ArrowPrimitiveType,
+    PrimitiveArray, StringArray,
 };
 use arrow_buffer::buffer::NullBuffer;
 use arrow_buffer::ArrowNativeType;
@@ -196,21 +196,20 @@ pub type UInt64DictionaryArray = DictionaryArray<UInt64Type>;
 /// Example from existing arrays:
 ///
 /// ```
+/// use std::sync::Arc;
 /// use arrow_array::{DictionaryArray, Int8Array, StringArray, types::Int8Type};
 /// // You can form your own DictionaryArray by providing the
 /// // values (dictionary) and keys (indexes into the dictionary):
 /// let values = StringArray::from_iter_values(["a", "b", "c"]);
 /// let keys = Int8Array::from_iter_values([0, 0, 1, 2]);
-/// let array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+/// let array = DictionaryArray::<Int8Type>::try_new(keys, Arc::new(values)).unwrap();
 /// let expected: DictionaryArray::<Int8Type> = vec!["a", "a", "b", "c"]
 ///    .into_iter()
 ///    .collect();
 /// assert_eq!(&array, &expected);
 /// ```
 pub struct DictionaryArray<K: ArrowDictionaryKeyType> {
-    /// Data of this dictionary. Note that this is _not_ compatible with the C Data interface,
-    /// as, in the current implementation, `values` below are the first child of this struct.
-    data: ArrayData,
+    data_type: DataType,
 
     /// The keys of this dictionary. These are constructed from the
     /// buffer and null bitmap of `data`.  Also, note that these do
@@ -228,7 +227,7 @@ pub struct DictionaryArray<K: ArrowDictionaryKeyType> {
 impl<K: ArrowDictionaryKeyType> Clone for DictionaryArray<K> {
     fn clone(&self) -> Self {
         Self {
-            data: self.data.clone(),
+            data_type: self.data_type.clone(),
             keys: self.keys.clone(),
             values: self.values.clone(),
             is_ordered: self.is_ordered,
@@ -239,33 +238,72 @@ impl<K: ArrowDictionaryKeyType> Clone for DictionaryArray<K> {
 impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
     /// Attempt to create a new DictionaryArray with a specified keys
     /// (indexes into the dictionary) and values (dictionary)
-    /// array. Returns an error if there are any keys that are outside
-    /// of the dictionary array.
+    /// array.
+    ///
+    /// # Panics
+    ///
+    /// Panics if [`Self::try_new`] returns an error
+    pub fn new(keys: PrimitiveArray<K>, values: ArrayRef) -> Self {
+        Self::try_new(keys, values).unwrap()
+    }
+
+    /// Attempt to create a new DictionaryArray with a specified keys
+    /// (indexes into the dictionary) and values (dictionary)
+    /// array.
+    ///
+    /// # Errors
+    ///
+    /// Returns an error if any `keys[i] >= values.len() || keys[i] < 0`
     pub fn try_new(
-        keys: &PrimitiveArray<K>,
-        values: &dyn Array,
+        keys: PrimitiveArray<K>,
+        values: ArrayRef,
     ) -> Result<Self, ArrowError> {
-        let dict_data_type = DataType::Dictionary(
+        let data_type = DataType::Dictionary(
             Box::new(keys.data_type().clone()),
             Box::new(values.data_type().clone()),
         );
 
-        // Note: This use the ArrayDataBuilder::build_unchecked and afterwards
-        // call the new function which only validates that the keys are in bounds.
-        let data = keys.to_data();
-        let builder = data
-            .into_builder()
-            .data_type(dict_data_type)
-            .add_child_data(values.to_data());
+        let zero = K::Native::usize_as(0);
+        let values_len = values.len();
 
-        // Safety: `validate` ensures key type is correct, and
-        //  `validate_values` ensures all offsets are within range
-        let array = unsafe { builder.build_unchecked() };
+        if let Some((idx, v)) = keys.values().iter().enumerate().find(|(idx, v)| {
+            (v.is_lt(zero) || v.as_usize() >= values_len) && keys.is_valid(*idx)
+        }) {
+            return Err(ArrowError::InvalidArgumentError(format!(
+                "Invalid dictionary key {v:?} at index {idx}, expected 0 <= key < {values_len}",
+            )));
+        }
 
-        array.validate()?;
-        array.validate_values()?;
+        Ok(Self {
+            data_type,
+            keys,
+            values,
+            is_ordered: false,
+        })
+    }
 
-        Ok(array.into())
+    /// Create a new [`DictionaryArray`] without performing validation
+    ///
+    /// # Safety
+    ///
+    /// Safe provided [`Self::try_new`] would not return an error
+    pub unsafe fn new_unchecked(keys: PrimitiveArray<K>, values: ArrayRef) -> Self {
+        let data_type = DataType::Dictionary(
+            Box::new(keys.data_type().clone()),
+            Box::new(values.data_type().clone()),
+        );
+
+        Self {
+            data_type,
+            keys,
+            values,
+            is_ordered: false,
+        }
+    }
+
+    /// Deconstruct this array into its constituent parts
+    pub fn into_parts(self) -> (PrimitiveArray<K>, ArrayRef) {
+        (self.keys, self.values)
     }
 
     /// Return an array view of the keys of this dictionary as a PrimitiveArray.
@@ -325,8 +363,12 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
 
     /// Returns a zero-copy slice of this array with the indicated offset and length.
     pub fn slice(&self, offset: usize, length: usize) -> Self {
-        // TODO: Slice buffers directly (#3880)
-        self.data.slice(offset, length).into()
+        Self {
+            data_type: self.data_type.clone(),
+            keys: self.keys.slice(offset, length),
+            values: self.values.clone(),
+            is_ordered: self.is_ordered,
+        }
     }
 
     /// Downcast this dictionary to a [`TypedDictionaryArray`]
@@ -360,9 +402,9 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
     /// Panics if `values` has a length less than the current values
     ///
     /// ```
-    /// use arrow_array::builder::PrimitiveDictionaryBuilder;
-    /// use arrow_array::{Int8Array, Int64Array, ArrayAccessor};
-    /// use arrow_array::types::{Int32Type, Int8Type};
+    /// # use arrow_array::builder::PrimitiveDictionaryBuilder;
+    /// # use arrow_array::{Int8Array, Int64Array, ArrayAccessor};
+    /// # use arrow_array::types::{Int32Type, Int8Type};
     ///
     /// // Construct a Dict(Int32, Int8)
     /// let mut builder = PrimitiveDictionaryBuilder::<Int32Type, Int8Type>::with_capacity(2, 200);
@@ -390,8 +432,7 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
         assert!(values.len() >= self.values.len());
 
         let builder = self
-            .data
-            .clone()
+            .to_data()
             .into_builder()
             .data_type(DataType::Dictionary(
                 Box::new(K::DATA_TYPE),
@@ -419,7 +460,6 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
         let key_array = self.keys().clone();
         let value_array = self.values().as_primitive::<V>().clone();
 
-        drop(self.data);
         drop(self.keys);
         drop(self.values);
 
@@ -431,13 +471,13 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
                 PrimitiveDictionaryBuilder::new_from_builders(key_builder, value_builder)
             }),
             (Err(key_array), Ok(mut value_builder)) => {
-                Err(Self::try_new(&key_array, &value_builder.finish()).unwrap())
+                Err(Self::try_new(key_array, Arc::new(value_builder.finish())).unwrap())
             }
             (Ok(mut key_builder), Err(value_array)) => {
-                Err(Self::try_new(&key_builder.finish(), &value_array).unwrap())
+                Err(Self::try_new(key_builder.finish(), Arc::new(value_array)).unwrap())
             }
             (Err(key_array), Err(value_array)) => {
-                Err(Self::try_new(&key_array, &value_array).unwrap())
+                Err(Self::try_new(key_array, Arc::new(value_array)).unwrap())
             }
         }
     }
@@ -453,13 +493,12 @@ impl<K: ArrowDictionaryKeyType> DictionaryArray<K> {
     /// or this function may panic.
     /// # Example
     /// ```
-    /// use arrow_array::{Array, ArrayAccessor, DictionaryArray, StringArray, types::{Int8Type, Int32Type}};
-    /// use arrow_array::{Int8Array, Int32Array};
+    /// # use std::sync::Arc;
+    /// # use arrow_array::{Array, ArrayAccessor, DictionaryArray, StringArray, types::{Int8Type, Int32Type}};
+    /// # use arrow_array::{Int8Array, Int32Array};
     /// let values = Int32Array::from(vec![Some(10), Some(20), None]);
     /// let keys = Int8Array::from_iter_values([0, 0, 1, 2]);
-    /// let dictionary = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
-    /// drop(keys);
-    /// drop(values);
+    /// let dictionary = DictionaryArray::<Int8Type>::try_new(keys, Arc::new(values)).unwrap();
     /// let c = dictionary.unary_mut::<_, Int32Type>(|x| x + 1).unwrap();
     /// let typed = c.downcast_dict::<Int32Array>().unwrap();
     /// assert_eq!(typed.value(0), 11);
@@ -504,20 +543,22 @@ impl<T: ArrowDictionaryKeyType> From<ArrayData> for DictionaryArray<T> {
                 key_data_type
             );
 
+            let values = make_array(data.child_data()[0].clone());
+            let data_type = data.data_type().clone();
+
             // create a zero-copy of the keys' data
             // SAFETY:
             // ArrayData is valid and verified type above
 
             let keys = PrimitiveArray::<T>::from(unsafe {
-                data.clone()
-                    .into_builder()
+                data.into_builder()
                     .data_type(T::DATA_TYPE)
                     .child_data(vec![])
                     .build_unchecked()
             });
-            let values = make_array(data.child_data()[0].clone());
+
             Self {
-                data,
+                data_type,
                 keys,
                 values,
                 is_ordered: false,
@@ -530,7 +571,14 @@ impl<T: ArrowDictionaryKeyType> From<ArrayData> for DictionaryArray<T> {
 
 impl<T: ArrowDictionaryKeyType> From<DictionaryArray<T>> for ArrayData {
     fn from(array: DictionaryArray<T>) -> Self {
-        array.data
+        let builder = array
+            .keys
+            .into_data()
+            .into_builder()
+            .data_type(array.data_type)
+            .child_data(vec![array.values.to_data()]);
+
+        unsafe { builder.build_unchecked() }
     }
 }
 
@@ -594,24 +642,46 @@ impl<T: ArrowDictionaryKeyType> Array for DictionaryArray<T> {
         self
     }
 
-    fn data(&self) -> &ArrayData {
-        &self.data
-    }
-
     fn to_data(&self) -> ArrayData {
-        self.data.clone()
+        self.clone().into()
     }
 
     fn into_data(self) -> ArrayData {
         self.into()
     }
 
+    fn data_type(&self) -> &DataType {
+        &self.data_type
+    }
+
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
         Arc::new(self.slice(offset, length))
     }
 
+    fn len(&self) -> usize {
+        self.keys.len()
+    }
+
+    fn is_empty(&self) -> bool {
+        self.keys.is_empty()
+    }
+
+    fn offset(&self) -> usize {
+        self.keys.offset()
+    }
+
     fn nulls(&self) -> Option<&NullBuffer> {
-        self.data.nulls()
+        self.keys.nulls()
+    }
+
+    fn get_buffer_memory_size(&self) -> usize {
+        self.keys.get_buffer_memory_size() + self.values.get_buffer_memory_size()
+    }
+
+    fn get_array_memory_size(&self) -> usize {
+        std::mem::size_of::<Self>()
+            + self.keys.get_buffer_memory_size()
+            + self.values.get_array_memory_size()
     }
 }
 
@@ -685,10 +755,6 @@ impl<'a, K: ArrowDictionaryKeyType, V: Sync> Array for TypedDictionaryArray<'a,
         self.dictionary
     }
 
-    fn data(&self) -> &ArrayData {
-        &self.dictionary.data
-    }
-
     fn to_data(&self) -> ArrayData {
         self.dictionary.to_data()
     }
@@ -697,13 +763,37 @@ impl<'a, K: ArrowDictionaryKeyType, V: Sync> Array for TypedDictionaryArray<'a,
         self.dictionary.into_data()
     }
 
+    fn data_type(&self) -> &DataType {
+        self.dictionary.data_type()
+    }
+
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
         Arc::new(self.dictionary.slice(offset, length))
     }
 
+    fn len(&self) -> usize {
+        self.dictionary.len()
+    }
+
+    fn is_empty(&self) -> bool {
+        self.dictionary.is_empty()
+    }
+
+    fn offset(&self) -> usize {
+        self.dictionary.offset()
+    }
+
     fn nulls(&self) -> Option<&NullBuffer> {
         self.dictionary.nulls()
     }
+
+    fn get_buffer_memory_size(&self) -> usize {
+        self.dictionary.get_buffer_memory_size()
+    }
+
+    fn get_array_memory_size(&self) -> usize {
+        self.dictionary.get_array_memory_size()
+    }
 }
 
 impl<'a, K, V> IntoIterator for TypedDictionaryArray<'a, K, V>
@@ -756,7 +846,7 @@ mod tests {
     use super::*;
     use crate::builder::PrimitiveDictionaryBuilder;
     use crate::cast::as_dictionary_array;
-    use crate::types::{Int16Type, Int32Type, Int8Type, UInt32Type, UInt8Type};
+    use crate::types::{Int32Type, Int8Type, UInt32Type, UInt8Type};
     use crate::{Int16Array, Int32Array, Int8Array};
     use arrow_buffer::{Buffer, ToByteSlice};
     use std::sync::Arc;
@@ -923,7 +1013,7 @@ mod tests {
         let keys = Int16Array::from_iter_values([2_i16, 3, 4]);
 
         // Construct a dictionary array from the above two
-        let dict_array = DictionaryArray::<Int16Type>::try_new(&keys, &values).unwrap();
+        let dict_array = DictionaryArray::new(keys, Arc::new(values));
 
         let mut key_iter = dict_array.keys_iter();
         assert_eq!(2, key_iter.next().unwrap().unwrap());
@@ -970,7 +1060,7 @@ mod tests {
         let keys = Int8Array::from(vec![Some(2), None, Some(1)]);
         let values = StringArray::from(vec!["foo", "bar", "baz", "blarg"]);
 
-        let array = DictionaryArray::try_new(&keys, &values).unwrap();
+        let array = DictionaryArray::new(keys, Arc::new(values));
         assert_eq!(array.key(0), Some(2));
         assert_eq!(array.key(1), None);
         assert_eq!(array.key(2), Some(1));
@@ -983,7 +1073,7 @@ mod tests {
             .collect();
         let keys: Int32Array = [Some(0), Some(2), None, Some(1)].into_iter().collect();
 
-        let array = DictionaryArray::<Int32Type>::try_new(&keys, &values).unwrap();
+        let array = DictionaryArray::new(keys, Arc::new(values));
         assert_eq!(array.keys().data_type(), &DataType::Int32);
         assert_eq!(array.values().data_type(), &DataType::Utf8);
 
@@ -1006,23 +1096,23 @@ mod tests {
 
     #[test]
     #[should_panic(
-        expected = "Value at position 1 out of bounds: 3 (should be in [0, 1])"
+        expected = "Invalid dictionary key 3 at index 1, expected 0 <= key < 2"
     )]
     fn test_try_new_index_too_large() {
         let values: StringArray = [Some("foo"), Some("bar")].into_iter().collect();
         // dictionary only has 2 values, so offset 3 is out of bounds
         let keys: Int32Array = [Some(0), Some(3)].into_iter().collect();
-        DictionaryArray::<Int32Type>::try_new(&keys, &values).unwrap();
+        DictionaryArray::new(keys, Arc::new(values));
     }
 
     #[test]
     #[should_panic(
-        expected = "Value at position 0 out of bounds: -100 (should be in [0, 1])"
+        expected = "Invalid dictionary key -100 at index 0, expected 0 <= key < 2"
     )]
     fn test_try_new_index_too_small() {
         let values: StringArray = [Some("foo"), Some("bar")].into_iter().collect();
         let keys: Int32Array = [Some(-100)].into_iter().collect();
-        DictionaryArray::<Int32Type>::try_new(&keys, &values).unwrap();
+        DictionaryArray::new(keys, Arc::new(values));
     }
 
     #[test]
@@ -1039,14 +1129,12 @@ mod tests {
         let values = Int32Array::from_iter_values([10_i32, 12, 15]);
         let keys = Int8Array::from_iter_values([1_i8, 0, 2, 0]);
 
-        let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+        let dict_array = DictionaryArray::new(keys, Arc::new(values));
 
         let boxed: ArrayRef = Arc::new(dict_array);
         let col: DictionaryArray<Int8Type> = as_dictionary_array(&boxed).clone();
 
         drop(boxed);
-        drop(keys);
-        drop(values);
 
         let mut builder = col.into_primitive_dict_builder::<Int32Type>().unwrap();
 
@@ -1060,7 +1148,7 @@ mod tests {
         let values = Int32Array::from_iter_values([4_i32, 2, 1]);
         let keys = Int8Array::from_iter_values([1_i8, 0, 2, 0]);
 
-        let expected = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+        let expected = DictionaryArray::new(keys, Arc::new(values));
 
         let new_array = builder.finish();
         assert_eq!(expected, new_array);
@@ -1071,7 +1159,7 @@ mod tests {
         let values = Int32Array::from_iter_values([10_i32, 12, 15]);
         let keys = Int8Array::from_iter_values([1_i8, 0, 2, 0]);
 
-        let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+        let dict_array = DictionaryArray::new(keys, Arc::new(values));
 
         let boxed: ArrayRef = Arc::new(dict_array);
 
@@ -1084,7 +1172,7 @@ mod tests {
         let values = Int32Array::from_iter_values([10_i32, 12, 15]);
         let keys = Int8Array::from_iter_values([1_i8, 0, 2, 0]);
 
-        let expected = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+        let expected = DictionaryArray::new(keys, Arc::new(values));
         assert_eq!(expected, returned);
     }
 }
