@@ -19,7 +19,7 @@ use crate::array::print_long_array;
 use crate::builder::{FixedSizeListBuilder, PrimitiveBuilder};
 use crate::{make_array, Array, ArrayAccessor, ArrayRef, ArrowPrimitiveType};
 use arrow_buffer::buffer::NullBuffer;
-use arrow_data::ArrayData;
+use arrow_data::{ArrayData, ArrayDataBuilder};
 use arrow_schema::DataType;
 use std::any::Any;
 use std::sync::Arc;
@@ -64,9 +64,11 @@ use std::sync::Arc;
 /// [crate::array::FixedSizeBinaryArray]
 #[derive(Clone)]
 pub struct FixedSizeListArray {
-    data: ArrayData,
+    data_type: DataType, // Must be DataType::FixedSizeList(value_length)
     values: ArrayRef,
-    length: i32,
+    nulls: Option<NullBuffer>,
+    value_length: i32,
+    len: usize,
 }
 
 impl FixedSizeListArray {
@@ -91,7 +93,7 @@ impl FixedSizeListArray {
     /// Note this doesn't do any bound checking, for performance reason.
     #[inline]
     pub fn value_offset(&self, i: usize) -> i32 {
-        self.value_offset_at(self.data.offset() + i)
+        self.value_offset_at(i)
     }
 
     /// Returns the length for an element.
@@ -99,18 +101,29 @@ impl FixedSizeListArray {
     /// All elements have the same length as the array is a fixed size.
     #[inline]
     pub const fn value_length(&self) -> i32 {
-        self.length
+        self.value_length
     }
 
     #[inline]
     const fn value_offset_at(&self, i: usize) -> i32 {
-        i as i32 * self.length
+        i as i32 * self.value_length
     }
 
     /// Returns a zero-copy slice of this array with the indicated offset and length.
-    pub fn slice(&self, offset: usize, length: usize) -> Self {
-        // TODO: Slice buffers directly (#3880)
-        self.data.slice(offset, length).into()
+    pub fn slice(&self, offset: usize, len: usize) -> Self {
+        assert!(
+            offset.saturating_add(len) <= self.len,
+            "the length + offset of the sliced FixedSizeListArray cannot exceed the existing length"
+        );
+        let size = self.value_length as usize;
+
+        Self {
+            data_type: self.data_type.clone(),
+            values: self.values.slice(offset * size, len * size),
+            nulls: self.nulls.as_ref().map(|n| n.slice(offset, len)),
+            value_length: self.value_length,
+            len,
+        }
     }
 
     /// Creates a [`FixedSizeListArray`] from an iterator of primitive values
@@ -163,45 +176,35 @@ impl FixedSizeListArray {
 
 impl From<ArrayData> for FixedSizeListArray {
     fn from(data: ArrayData) -> Self {
-        assert_eq!(
-            data.buffers().len(),
-            0,
-            "FixedSizeListArray data should not contain a buffer for value offsets"
-        );
-        assert_eq!(
-            data.child_data().len(),
-            1,
-            "FixedSizeListArray should contain a single child array (values array)"
-        );
-        let values = make_array(data.child_data()[0].clone());
-        let length = match data.data_type() {
-            DataType::FixedSizeList(_, len) => {
-                if *len > 0 {
-                    // check that child data is multiple of length
-                    assert_eq!(
-                        values.len() % *len as usize,
-                        0,
-                        "FixedSizeListArray child array length should be a multiple of {len}"
-                    );
-                }
-
-                *len
-            }
+        let value_length = match data.data_type() {
+            DataType::FixedSizeList(_, len) => *len,
             _ => {
                 panic!("FixedSizeListArray data should contain a FixedSizeList data type")
             }
         };
+
+        let size = value_length as usize;
+        let values = make_array(
+            data.child_data()[0].slice(data.offset() * size, data.len() * size),
+        );
         Self {
-            data,
+            data_type: data.data_type().clone(),
             values,
-            length,
+            nulls: data.nulls().cloned(),
+            value_length,
+            len: data.len(),
         }
     }
 }
 
 impl From<FixedSizeListArray> for ArrayData {
     fn from(array: FixedSizeListArray) -> Self {
-        array.data
+        let builder = ArrayDataBuilder::new(array.data_type)
+            .len(array.len)
+            .nulls(array.nulls)
+            .child_data(vec![array.values.to_data()]);
+
+        unsafe { builder.build_unchecked() }
     }
 }
 
@@ -210,24 +213,52 @@ impl Array for FixedSizeListArray {
         self
     }
 
-    fn data(&self) -> &ArrayData {
-        &self.data
-    }
-
     fn to_data(&self) -> ArrayData {
-        self.data.clone()
+        self.clone().into()
     }
 
     fn into_data(self) -> ArrayData {
         self.into()
     }
 
+    fn data_type(&self) -> &DataType {
+        &self.data_type
+    }
+
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
         Arc::new(self.slice(offset, length))
     }
 
+    fn len(&self) -> usize {
+        self.len
+    }
+
+    fn is_empty(&self) -> bool {
+        self.len == 0
+    }
+
+    fn offset(&self) -> usize {
+        0
+    }
+
     fn nulls(&self) -> Option<&NullBuffer> {
-        self.data.nulls()
+        self.nulls.as_ref()
+    }
+
+    fn get_buffer_memory_size(&self) -> usize {
+        let mut size = self.values.get_buffer_memory_size();
+        if let Some(n) = self.nulls.as_ref() {
+            size += n.buffer().capacity();
+        }
+        size
+    }
+
+    fn get_array_memory_size(&self) -> usize {
+        let mut size = std::mem::size_of::<Self>() + self.values.get_array_memory_size();
+        if let Some(n) = self.nulls.as_ref() {
+            size += n.buffer().capacity();
+        }
+        size
     }
 }
 
@@ -258,7 +289,6 @@ mod tests {
     use super::*;
     use crate::cast::AsArray;
     use crate::types::Int32Type;
-    use crate::Int32Array;
     use arrow_buffer::{bit_util, Buffer};
     use arrow_schema::Field;
 
@@ -289,15 +319,7 @@ mod tests {
         assert_eq!(0, list_array.null_count());
         assert_eq!(6, list_array.value_offset(2));
         assert_eq!(3, list_array.value_length());
-        assert_eq!(
-            0,
-            list_array
-                .value(0)
-                .as_any()
-                .downcast_ref::<Int32Array>()
-                .unwrap()
-                .value(0)
-        );
+        assert_eq!(0, list_array.value(0).as_primitive::<Int32Type>().value(0));
         for i in 0..3 {
             assert!(list_array.is_valid(i));
             assert!(!list_array.is_null(i));
@@ -305,26 +327,24 @@ mod tests {
 
         // Now test with a non-zero offset
         let list_data = ArrayData::builder(list_data_type)
-            .len(3)
+            .len(2)
             .offset(1)
             .add_child_data(value_data.clone())
             .build()
             .unwrap();
         let list_array = FixedSizeListArray::from(list_data);
 
-        assert_eq!(value_data, list_array.values().to_data());
+        assert_eq!(value_data.slice(3, 6), list_array.values().to_data());
         assert_eq!(DataType::Int32, list_array.value_type());
-        assert_eq!(3, list_array.len());
+        assert_eq!(2, list_array.len());
         assert_eq!(0, list_array.null_count());
         assert_eq!(3, list_array.value(0).as_primitive::<Int32Type>().value(0));
-        assert_eq!(6, list_array.value_offset(1));
+        assert_eq!(3, list_array.value_offset(1));
         assert_eq!(3, list_array.value_length());
     }
 
     #[test]
-    #[should_panic(
-        expected = "FixedSizeListArray child array length should be a multiple of 3"
-    )]
+    #[should_panic(expected = "assertion failed: (offset + length) <= self.len()")]
     // Different error messages, so skip for now
     // https://github.com/apache/arrow-rs/issues/1545
     #[cfg(not(feature = "force_validate"))]
@@ -389,11 +409,10 @@ mod tests {
 
         let sliced_array = list_array.slice(1, 4);
         assert_eq!(4, sliced_array.len());
-        assert_eq!(1, sliced_array.offset());
         assert_eq!(2, sliced_array.null_count());
 
         for i in 0..sliced_array.len() {
-            if bit_util::get_bit(&null_bits, sliced_array.offset() + i) {
+            if bit_util::get_bit(&null_bits, 1 + i) {
                 assert!(sliced_array.is_valid(i));
             } else {
                 assert!(sliced_array.is_null(i));
@@ -406,12 +425,14 @@ mod tests {
             .downcast_ref::<FixedSizeListArray>()
             .unwrap();
         assert_eq!(2, sliced_list_array.value_length());
-        assert_eq!(6, sliced_list_array.value_offset(2));
-        assert_eq!(8, sliced_list_array.value_offset(3));
+        assert_eq!(4, sliced_list_array.value_offset(2));
+        assert_eq!(6, sliced_list_array.value_offset(3));
     }
 
     #[test]
-    #[should_panic(expected = "assertion failed: (offset + length) <= self.len()")]
+    #[should_panic(
+        expected = "the offset of the new Buffer cannot exceed the existing length"
+    )]
     fn test_fixed_size_list_array_index_out_of_bound() {
         // Construct a value array
         let value_data = ArrayData::builder(DataType::Int32)
