@@ -18,7 +18,7 @@
 use crate::array::{get_offsets, print_long_array};
 use crate::{make_array, Array, ArrayRef, ListArray, StringArray, StructArray};
 use arrow_buffer::{ArrowNativeType, Buffer, NullBuffer, OffsetBuffer, ToByteSlice};
-use arrow_data::ArrayData;
+use arrow_data::{ArrayData, ArrayDataBuilder};
 use arrow_schema::{ArrowError, DataType, Field};
 use std::any::Any;
 use std::sync::Arc;
@@ -30,49 +30,54 @@ use std::sync::Arc;
 /// [StructArray] with 2 child fields.
 #[derive(Clone)]
 pub struct MapArray {
-    data: ArrayData,
+    data_type: DataType,
+    nulls: Option<NullBuffer>,
     /// The [`StructArray`] that is the direct child of this array
-    entries: ArrayRef,
-    /// The first child of `entries`, the "keys" of this MapArray
-    keys: ArrayRef,
-    /// The second child of `entries`, the "values" of this MapArray
-    values: ArrayRef,
+    entries: StructArray,
     /// The start and end offsets of each entry
     value_offsets: OffsetBuffer<i32>,
 }
 
 impl MapArray {
-    /// Returns a reference to the keys of this map.
-    pub fn keys(&self) -> &ArrayRef {
-        &self.keys
+    /// Returns a reference to the offsets of this map
+    ///
+    /// Unlike [`Self::value_offsets`] this returns the [`OffsetBuffer`]
+    /// allowing for zero-copy cloning
+    #[inline]
+    pub fn offsets(&self) -> &OffsetBuffer<i32> {
+        &self.value_offsets
     }
 
-    /// Returns a reference to the values of this map.
+    /// Returns a reference to the keys of this map
+    pub fn keys(&self) -> &ArrayRef {
+        self.entries.column(0)
+    }
+
+    /// Returns a reference to the values of this map
     pub fn values(&self) -> &ArrayRef {
-        &self.values
+        self.entries.column(1)
     }
 
     /// Returns a reference to the [`StructArray`] entries of this map
-    pub fn entries(&self) -> &ArrayRef {
+    pub fn entries(&self) -> &StructArray {
         &self.entries
     }
 
     /// Returns the data type of the map's keys.
     pub fn key_type(&self) -> &DataType {
-        self.keys.data_type()
+        self.keys().data_type()
     }
 
     /// Returns the data type of the map's values.
     pub fn value_type(&self) -> &DataType {
-        self.values.data_type()
+        self.values().data_type()
     }
 
     /// Returns ith value of this map array.
     ///
-    /// This is a [`StructArray`] containing two fields
     /// # Safety
     /// Caller must ensure that the index is within the array bounds
-    pub unsafe fn value_unchecked(&self, i: usize) -> ArrayRef {
+    pub unsafe fn value_unchecked(&self, i: usize) -> StructArray {
         let end = *self.value_offsets().get_unchecked(i + 1);
         let start = *self.value_offsets().get_unchecked(i);
         self.entries
@@ -82,7 +87,7 @@ impl MapArray {
     /// Returns ith value of this map array.
     ///
     /// This is a [`StructArray`] containing two fields
-    pub fn value(&self, i: usize) -> ArrayRef {
+    pub fn value(&self, i: usize) -> StructArray {
         let end = self.value_offsets()[i + 1] as usize;
         let start = self.value_offsets()[i] as usize;
         self.entries.slice(start, end - start)
@@ -103,8 +108,12 @@ impl MapArray {
 
     /// Returns a zero-copy slice of this array with the indicated offset and length.
     pub fn slice(&self, offset: usize, length: usize) -> Self {
-        // TODO: Slice buffers directly (#3880)
-        self.data.slice(offset, length).into()
+        Self {
+            data_type: self.data_type.clone(),
+            nulls: self.nulls.as_ref().map(|n| n.slice(offset, length)),
+            entries: self.entries.clone(),
+            value_offsets: self.value_offsets.slice(offset, length),
+        }
     }
 }
 
@@ -117,7 +126,14 @@ impl From<ArrayData> for MapArray {
 
 impl From<MapArray> for ArrayData {
     fn from(array: MapArray) -> Self {
-        array.data
+        let len = array.len();
+        let builder = ArrayDataBuilder::new(array.data_type)
+            .len(len)
+            .nulls(array.nulls)
+            .buffers(vec![array.value_offsets.into_inner().into_inner()])
+            .child_data(vec![array.entries.to_data()]);
+
+        unsafe { builder.build_unchecked() }
     }
 }
 
@@ -158,20 +174,16 @@ impl MapArray {
                 entries.data_type()
             )));
         }
-
-        let keys = make_array(entries.child_data()[0].clone());
-        let values = make_array(entries.child_data()[1].clone());
-        let entries = make_array(entries);
+        let entries = entries.into();
 
         // SAFETY:
         // ArrayData is valid, and verified type above
         let value_offsets = unsafe { get_offsets(&data) };
 
         Ok(Self {
-            data,
+            data_type: data.data_type().clone(),
+            nulls: data.nulls().cloned(),
             entries,
-            keys,
-            values,
             value_offsets,
         })
     }
@@ -220,34 +232,54 @@ impl Array for MapArray {
         self
     }
 
-    fn data(&self) -> &ArrayData {
-        &self.data
-    }
-
     fn to_data(&self) -> ArrayData {
-        self.data.clone()
+        self.clone().into_data()
     }
 
     fn into_data(self) -> ArrayData {
         self.into()
     }
 
+    fn data_type(&self) -> &DataType {
+        &self.data_type
+    }
+
     fn slice(&self, offset: usize, length: usize) -> ArrayRef {
         Arc::new(self.slice(offset, length))
     }
 
+    fn len(&self) -> usize {
+        self.value_offsets.len() - 1
+    }
+
+    fn is_empty(&self) -> bool {
+        self.value_offsets.len() <= 1
+    }
+
+    fn offset(&self) -> usize {
+        0
+    }
+
     fn nulls(&self) -> Option<&NullBuffer> {
-        self.data.nulls()
+        self.nulls.as_ref()
     }
 
-    /// Returns the total number of bytes of memory occupied by the buffers owned by this [MapArray].
     fn get_buffer_memory_size(&self) -> usize {
-        self.data.get_buffer_memory_size()
+        let mut size = self.entries.get_buffer_memory_size();
+        size += self.value_offsets.inner().inner().capacity();
+        if let Some(n) = self.nulls.as_ref() {
+            size += n.buffer().capacity();
+        }
+        size
     }
 
-    /// Returns the total number of bytes of memory occupied physically by this [MapArray].
     fn get_array_memory_size(&self) -> usize {
-        self.data.get_array_memory_size() + std::mem::size_of_val(self)
+        let mut size = std::mem::size_of::<Self>() + self.entries.get_array_memory_size();
+        size += self.value_offsets.inner().inner().capacity();
+        if let Some(n) = self.nulls.as_ref() {
+            size += n.buffer().capacity();
+        }
+        size
     }
 }
 
