@@ -1380,8 +1380,7 @@ mod tests {
 
         let array12_values = StringArray::from(vec!["x", "yy", "zzz"]);
         let array12_keys = Int8Array::from_iter_values([1, 1, 2]);
-        let array12 =
-            DictionaryArray::<Int8Type>::try_new(&array12_keys, &array12_values).unwrap();
+        let array12 = DictionaryArray::new(array12_keys, Arc::new(array12_values));
 
         let array13 = StringArray::from(vec!["a", "bb", "ccc"]);
 
@@ -1693,13 +1692,12 @@ mod tests {
     #[test]
     fn test_roundtrip_stream_nested_dict_of_map_of_dict() {
         let values = StringArray::from(vec![Some("a"), None, Some("b"), Some("c")]);
+        let values = Arc::new(values) as ArrayRef;
         let value_dict_keys = Int8Array::from_iter_values([0, 1, 1, 2, 3, 1]);
-        let value_dict_array =
-            DictionaryArray::<Int8Type>::try_new(&value_dict_keys, &values).unwrap();
+        let value_dict_array = DictionaryArray::new(value_dict_keys, values.clone());
 
         let key_dict_keys = Int8Array::from_iter_values([0, 0, 2, 1, 1, 3]);
-        let key_dict_array =
-            DictionaryArray::<Int8Type>::try_new(&key_dict_keys, &values).unwrap();
+        let key_dict_array = DictionaryArray::new(key_dict_keys, values);
 
         let keys_field = Field::new_dict(
             "keys",
@@ -1738,8 +1736,7 @@ mod tests {
         let map_array = MapArray::from(map_data);
 
         let dict_keys = Int8Array::from_iter_values([0, 1, 1, 2, 2, 1]);
-        let dict_dict_array =
-            DictionaryArray::<Int8Type>::try_new(&dict_keys, &map_array).unwrap();
+        let dict_dict_array = DictionaryArray::new(dict_keys, Arc::new(map_array));
 
         let schema = Arc::new(Schema::new(vec![Field::new(
             "f1",
@@ -1761,7 +1758,7 @@ mod tests {
     ) {
         let values = StringArray::from(vec![Some("a"), None, Some("c"), None]);
         let keys = Int8Array::from_iter_values([0, 0, 1, 2, 0, 1, 3]);
-        let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
+        let dict_array = DictionaryArray::new(keys, Arc::new(values));
         let dict_data = dict_array.to_data();
 
         let value_offsets = Buffer::from_slice_ref(offsets);
@@ -1775,8 +1772,7 @@ mod tests {
         let list_array = GenericListArray::<OffsetSize>::from(list_data);
 
         let keys_for_dict = Int8Array::from_iter_values([0, 3, 0, 1, 1, 2, 0, 1, 3]);
-        let dict_dict_array =
-            DictionaryArray::<Int8Type>::try_new(&keys_for_dict, &list_array).unwrap();
+        let dict_dict_array = DictionaryArray::new(keys_for_dict, Arc::new(list_array));
 
         let schema = Arc::new(Schema::new(vec![Field::new(
             "f1",
@@ -1824,8 +1820,8 @@ mod tests {
     fn test_roundtrip_stream_dict_of_fixed_size_list_of_dict() {
         let values = StringArray::from(vec![Some("a"), None, Some("c"), None]);
         let keys = Int8Array::from_iter_values([0, 0, 1, 2, 0, 1, 3, 1, 2]);
-        let dict_array = DictionaryArray::<Int8Type>::try_new(&keys, &values).unwrap();
-        let dict_data = dict_array.to_data();
+        let dict_array = DictionaryArray::new(keys, Arc::new(values));
+        let dict_data = dict_array.into_data();
 
         let list_data_type = DataType::FixedSizeList(
             Arc::new(Field::new_dict(
@@ -1845,8 +1841,7 @@ mod tests {
         let list_array = FixedSizeListArray::from(list_data);
 
         let keys_for_dict = Int8Array::from_iter_values([0, 1, 0, 1, 1, 2, 0, 1, 2]);
-        let dict_dict_array =
-            DictionaryArray::<Int8Type>::try_new(&keys_for_dict, &list_array).unwrap();
+        let dict_dict_array = DictionaryArray::new(keys_for_dict, Arc::new(list_array));
 
         let schema = Arc::new(Schema::new(vec![Field::new(
             "f1",
