@@ -40,7 +40,9 @@ use arrow_buffer::{i256, Buffer};
 use arrow_cast::pretty::pretty_format_columns;
 use arrow_cast::{can_cast_types, cast};
 use arrow_data::ArrayData;
-use arrow_schema::{ArrowError, DataType, Field, IntervalUnit, TimeUnit, UnionMode};
+use arrow_schema::{
+    ArrowError, DataType, Field, Fields, IntervalUnit, TimeUnit, UnionFields, UnionMode,
+};
 use half::f16;
 use std::sync::Arc;
 
@@ -183,9 +185,7 @@ fn get_arrays_of_all_types() -> Vec<ArrayRef> {
         Arc::new(DurationMillisecondArray::from(vec![1000, 2000])),
         Arc::new(DurationMicrosecondArray::from(vec![1000, 2000])),
         Arc::new(DurationNanosecondArray::from(vec![1000, 2000])),
-        Arc::new(
-            create_decimal_array(vec![Some(1), Some(2), Some(3), None], 38, 0).unwrap(),
-        ),
+        Arc::new(create_decimal_array(vec![Some(1), Some(2), Some(3)], 38, 0).unwrap()),
         make_dictionary_primitive::<Int8Type, Decimal128Type>(vec![1, 2]),
         make_dictionary_primitive::<Int16Type, Decimal128Type>(vec![1, 2]),
         make_dictionary_primitive::<Int32Type, Decimal128Type>(vec![1, 2]),
@@ -239,7 +239,7 @@ fn make_fixed_size_list_array() -> FixedSizeListArray {
 
     // Construct a fixed size list array from the above two
     let list_data_type =
-        DataType::FixedSizeList(Box::new(Field::new("item", DataType::Int32, true)), 2);
+        DataType::FixedSizeList(Arc::new(Field::new("item", DataType::Int32, true)), 2);
     let list_data = ArrayData::builder(list_data_type)
         .len(5)
         .add_child_data(value_data)
@@ -273,7 +273,7 @@ fn make_list_array() -> ListArray {
 
     // Construct a list array from the above two
     let list_data_type =
-        DataType::List(Box::new(Field::new("item", DataType::Int32, true)));
+        DataType::List(Arc::new(Field::new("item", DataType::Int32, true)));
     let list_data = ArrayData::builder(list_data_type)
         .len(3)
         .add_buffer(value_offsets)
@@ -297,7 +297,7 @@ fn make_large_list_array() -> LargeListArray {
 
     // Construct a list array from the above two
     let list_data_type =
-        DataType::LargeList(Box::new(Field::new("item", DataType::Int32, true)));
+        DataType::LargeList(Arc::new(Field::new("item", DataType::Int32, true)));
     let list_data = ArrayData::builder(list_data_type)
         .len(3)
         .add_buffer(value_offsets)
@@ -350,7 +350,7 @@ fn create_decimal_array(
 // Get a selection of datatypes to try and cast to
 fn get_all_types() -> Vec<DataType> {
     use DataType::*;
-    let tz_name = String::from("+08:00");
+    let tz_name: Arc<str> = Arc::from("+08:00");
 
     let mut types = vec![
         Null,
@@ -388,26 +388,28 @@ fn get_all_types() -> Vec<DataType> {
         Interval(IntervalUnit::DayTime),
         Interval(IntervalUnit::MonthDayNano),
         Binary,
-        FixedSizeBinary(10),
+        FixedSizeBinary(3),
         LargeBinary,
         Utf8,
         LargeUtf8,
-        List(Box::new(Field::new("item", DataType::Int8, true))),
-        List(Box::new(Field::new("item", DataType::Utf8, true))),
-        FixedSizeList(Box::new(Field::new("item", DataType::Int8, true)), 10),
-        FixedSizeList(Box::new(Field::new("item", DataType::Utf8, false)), 10),
-        LargeList(Box::new(Field::new("item", DataType::Int8, true))),
-        LargeList(Box::new(Field::new("item", DataType::Utf8, false))),
-        Struct(vec![
+        List(Arc::new(Field::new("item", DataType::Int8, true))),
+        List(Arc::new(Field::new("item", DataType::Utf8, true))),
+        FixedSizeList(Arc::new(Field::new("item", DataType::Int8, true)), 10),
+        FixedSizeList(Arc::new(Field::new("item", DataType::Utf8, false)), 10),
+        LargeList(Arc::new(Field::new("item", DataType::Int8, true))),
+        LargeList(Arc::new(Field::new("item", DataType::Utf8, false))),
+        Struct(Fields::from(vec![
             Field::new("f1", DataType::Int32, true),
             Field::new("f2", DataType::Utf8, true),
-        ]),
+        ])),
         Union(
-            vec![
-                Field::new("f1", DataType::Int32, false),
-                Field::new("f2", DataType::Utf8, true),
-            ],
-            vec![0, 1],
+            UnionFields::new(
+                vec![0, 1],
+                vec![
+                    Field::new("f1", DataType::Int32, false),
+                    Field::new("f2", DataType::Utf8, true),
+                ],
+            ),
             UnionMode::Dense,
         ),
         Decimal128(38, 0),
